/*
//
//       Elvees Performance Platform
//           signal processing
//
*/

#pragma once

#include "eppdefs.h"

#ifdef __cplusplus
extern "C" {
#endif


/*
//  Name:       epps_GetLibraryVersion
//  Purpose:    gets the version of the library
//  Returns:    structure containing information about the current version of the epps library
//  Parameters: None
*/
EPPAPI( const epp_LibraryVersion*, epps_GetLibraryVersion, (void) )


/*
//  Name:        epps_Window
//  Purpose:     Generates window function
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  length <= 1
//  Parameters:
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
*/
EPPAPI(epp_Status, epps_WindowBartlett_f16, (epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, epps_WindowBartlett_f32, (epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, epps_WindowHann_f16, (epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, epps_WindowHann_f32, (epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, epps_WindowHamming_f16, (epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, epps_WindowHamming_f32, (epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, epps_WindowBlackmanHarris_f16, (epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, epps_WindowBlackmanHarris_f32, (epp_f32* pDst, epp_size length))
EPPAPI(epp_Status, epps_WindowFlattop_f16, (epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, epps_WindowFlattop_f32, (epp_f32* pDst, epp_size length))


/*
//  Name:        epps_WaveSin
//  Purpose:     Generates Sin wave
//  Context:
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    magn       The magnitude of the wave (the maximum value attained by the wave)
//    rfreq      The frequency of the tone relative to the sampling frequency.
//    pPhase     The phase of the wave. The returned value can be used to compute the next continuous data
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
//  Notes:
//    for real:  pDst[i] = magn * sin(IPP_2PI * rfreq * i + phase);
//    for cplx:  pDst[i].re = magn * sin(IPP_2PI * rfreq * i + phase);
//               pDst[i].im = magn * cos(IPP_2PI * rfreq * i + phase);
*/
EPPAPI(epp_Status, epps_WaveSin_f16, (epp_f64 magn, epp_f64 rfreq, epp_f64* pPhase, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, epps_WaveSin_f32, (epp_f64 magn, epp_f64 rfreq, epp_f64* pPhase, epp_f32* pDst, epp_size length))


/*
//  Name:        epps_WaveTriangle
//  Purpose:     Generates Triangle wave
//  Context:
//  Returns:     epp_Status
//    epp_StsOK           Ok
//    epp_StsBadParamsErr (rfreq < 0) || (rfreq >= 1)
//                        (asymetry <= - EPP_PI) || (asymetry >= EPP_PI)
//                        (phase < 0) || (phase >= EPP_2PI)
//  Parameters:
//    magn       The magnitude of the wave (the maximum value attained by the wave)
//    rfreq      The frequency of the tone relative to the sampling frequency.
//    asymetry   The asymetry of triangle, should be in (-Pi; Pi)
//    pPhase     The phase of the wave. The returned value can be used to compute the next continuous data
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
*/
EPPAPI(epp_Status, epps_WaveTriangle_f16, (epp_f64 magn, epp_f64 rfreq, epp_f64 asymetry, epp_f64* pPhase, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, epps_WaveTriangle_f32, (epp_f64 magn, epp_f64 rfreq, epp_f64 asymetry, epp_f64* pPhase, epp_f32* pDst, epp_size length))


/*
//  Name:        epps_WaveSquare
//  Purpose:     Generates Square wave
//  Context:
//  Returns:     epp_Status
//    epp_StsOK           Ok
//    epp_StsBadParamsErr (rfreq < 0) || (rfreq >= 1)
//                        (asymetry <= - EPP_PI) || (asymetry >= EPP_PI)
//                        (phase < 0) || (phase >= EPP_2PI)
//  Parameters:
//    magn       The magnitude of the wave (the maximum value attained by the wave)
//    rfreq      The frequency of the tone relative to the sampling frequency.
//    asymetry   The asymetry of wave, should be in (-Pi; Pi)
//    pPhase     The phase of the wave. The returned value can be used to compute the next continuous data
//    pDst       The destination vector data pointer.
//    length     The length of vectors.
*/
EPPAPI(epp_Status, epps_WaveSquare_f16, (epp_f64 magn, epp_f64 rfreq, epp_f64 asymetry, epp_f64* pPhase, epp_f16* pDst, epp_size length))
EPPAPI(epp_Status, epps_WaveSquare_f32, (epp_f64 magn, epp_f64 rfreq, epp_f64 asymetry, epp_f64* pPhase, epp_f32* pDst, epp_size length))


/*
//  Name:        epps_FilterFIR
//  Purpose:     Perform FIR filtering of input signal
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  tapsLength < 1
//  Parameters:
//    pSrc       The source signal data pointer.
//    pTaps      The pointer to filter's coefficients.
//    tapsLength The length of filter's coefficients.
//    pDst       The destination signal data pointer.
//    dstLength  The length of filtered signal.
//    pDelayLine The pointer to input and output delay line.
//  Notes:
//    If pDelayLine == NULL the delay line is not used, src should have
//      dstLength samples to the right of pSrc and tapsLength-1 samples to the left of pSrc,
//      dst[i] = SUM(taps[j]*src[i-tapsLength+1+j]), j=0..tapsLength-1, i=0..dstLength
//    If pDelayLine != NULL the delay line is used to calculate first tapsLength-1 dst samples,
//      the size of delay line should be 2*(tapsLength-1), first tapsLength-1 elements are real
//      delay line, the second tapsLength-1 elements are used as aux buffer,
//      src should have dstLength samples to the right of pSrc.
//      dst[i] is calculated as for pDelayLine == NULL but delay samles are used instead of
//      src samples for i-tapsLength+1+j<0.
//      On exit delay line is updated by src[dstLength-1-tapsLength-1]..src[dstLength-1] samples.
*/
EPPAPI(epp_Status, epps_FilterFIR_f16, (const epp_f16* pSrc, const epp_f16* pTaps, epp_size tapsLength, epp_f16* pDst, epp_size dstLength, epp_f16* pDelayLine))
EPPAPI(epp_Status, epps_FilterFIR_f32, (const epp_f32* pSrc, const epp_f32* pTaps, epp_size tapsLength, epp_f32* pDst, epp_size dstLength, epp_f32* pDelayLine))


/*
//  Name:        epps_FilterMedian
//  Purpose:     Perform Median filtering of input signal
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  winLength is even
//  Parameters:
//    pSrc       The source signal data pointer.
//    winLength  The length of window, should be odd.
//    pDst       The destination signal data pointer.
//    dstLength  The length of filtered signal.
//    pDelayLine The pointer to input and output delay line.
//  Notes:
//    If pDelayLine == NULL the delay line is not used, src should have
//      dstLength samples to the right of pSrc and winLength-1 samples to the left of pSrc,
//      dst[i] = Median(src[i-winLength+1]..src[i]), i=0..dstLength
//    If pDelayLine != NULL the delay line is used to calculate first winLength-1 dst samples,
//      the size of delay line should be 2*(winLength-1), first winLength-1 elements are real
//      delay line, the second winLength-1 elements are used as aux buffer,
//      src should have dstLength samples to the right of pSrc.
//      dst[i] is calculated as for pDelayLine == NULL but delay samles are used instead of
//      src samples for i-winLength+1+j<0.
//      On exit delay line is updated by src[dstLength-1-winLength-1]..src[dstLength-1] samples.
*/
EPPAPI(epp_Status, epps_FilterMedian_f16, (const epp_f16* pSrc, epp_size winLength, epp_f16* pDst, epp_size dstLength, epp_f16* pDelayLine))
EPPAPI(epp_Status, epps_FilterMedian_f32, (const epp_f32* pSrc, epp_size winLength, epp_f32* pDst, epp_size dstLength, epp_f32* pDelayLine))


/*
//  Name:        epps_FilterMax
//  Purpose:     Perform Max filtering of input signal
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  winLength < 1
//  Parameters:
//    pSrc       The source signal data pointer.
//    winLength  The length of window, should be odd.
//    pDst       The destination signal data pointer.
//    dstLength  The length of filtered signal.
//    pDelayLine The pointer to input and output delay line.
//  Notes:
//    If pDelayLine == NULL the delay line is not used, src should have
//      dstLength samples to the right of pSrc and winLength-1 samples to the left of pSrc,
//      dst[i] = Max(src[i-winLength+1]..src[i]), i=0..dstLength
//    If pDelayLine != NULL the delay line is used to calculate first winLength-1 dst samples,
//      the size of delay line should be 2*(winLength-1), first winLength-1 elements are real
//      delay line, the second winLength-1 elements are used as aux buffer,
//      src should have dstLength samples to the right of pSrc.
//      dst[i] is calculated as for pDelayLine == NULL but delay samles are used instead of
//      src samples for i-winLength+1+j<0.
//      On exit delay line is updated by src[dstLength-1-winLength-1]..src[dstLength-1] samples.
*/
EPPAPI(epp_Status, epps_FilterMax_f16, (const epp_f16* pSrc, epp_size winLength, epp_f16* pDst, epp_size dstLength, epp_f16* pDelayLine))
EPPAPI(epp_Status, epps_FilterMax_f32, (const epp_f32* pSrc, epp_size winLength, epp_f32* pDst, epp_size dstLength, epp_f32* pDelayLine))


/*
//  Name:        epps_FilterMin
//  Purpose:     Perform Min filtering of input signal
//  Returns:     epp_Status
//    epp_StsOK            Ok
//    epp_StsBadParamsErr  winLength < 1
//  Parameters:
//    pSrc       The source signal data pointer.
//    winLength  The length of window, should be odd.
//    pDst       The destination signal data pointer.
//    dstLength  The length of filtered signal.
//    pDelayLine The pointer to input and output delay line.
//  Notes:
//    If pDelayLine == NULL the delay line is not used, src should have
//      dstLength samples to the right of pSrc and winLength-1 samples to the left of pSrc,
//      dst[i] = Min(src[i-winLength+1]..src[i]), i=0..dstLength
//    If pDelayLine != NULL the delay line is used to calculate first winLength-1 dst samples,
//      the size of delay line should be 2*(winLength-1), first winLength-1 elements are real
//      delay line, the second winLength-1 elements are used as aux buffer,
//      src should have dstLength samples to the right of pSrc.
//      dst[i] is calculated as for pDelayLine == NULL but delay samles are used instead of
//      src samples for i-winLength+1+j<0.
//      On exit delay line is updated by src[dstLength-1-winLength-1]..src[dstLength-1] samples.
*/
EPPAPI(epp_Status, epps_FilterMin_f16, (const epp_f16* pSrc, epp_size winLength, epp_f16* pDst, epp_size dstLength, epp_f16* pDelayLine))
EPPAPI(epp_Status, epps_FilterMin_f32, (const epp_f32* pSrc, epp_size winLength, epp_f32* pDst, epp_size dstLength, epp_f32* pDelayLine))


#ifdef __cplusplus
}
#endif
